# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Mapping, cast
from typing_extensions import Literal

import httpx

from .chunks import (
    ChunksResource,
    AsyncChunksResource,
    ChunksResourceWithRawResponse,
    AsyncChunksResourceWithRawResponse,
    ChunksResourceWithStreamingResponse,
    AsyncChunksResourceWithStreamingResponse,
)
from ...types import (
    knowledge_base_list_params,
    knowledge_base_query_params,
    knowledge_base_create_params,
    knowledge_base_retrieve_params,
    knowledge_base_upload_files_params,
)
from .uploads import (
    UploadsResource,
    AsyncUploadsResource,
    UploadsResourceWithRawResponse,
    AsyncUploadsResourceWithRawResponse,
    UploadsResourceWithStreamingResponse,
    AsyncUploadsResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ..._utils import (
    extract_files,
    maybe_transform,
    deepcopy_minimal,
    async_maybe_transform,
)
from ..._compat import cached_property
from .artifacts import (
    ArtifactsResource,
    AsyncArtifactsResource,
    ArtifactsResourceWithRawResponse,
    AsyncArtifactsResourceWithRawResponse,
    ArtifactsResourceWithStreamingResponse,
    AsyncArtifactsResourceWithStreamingResponse,
)
from .async_jobs import (
    AsyncJobsResource,
    AsyncAsyncJobsResource,
    AsyncJobsResourceWithRawResponse,
    AsyncAsyncJobsResourceWithRawResponse,
    AsyncJobsResourceWithStreamingResponse,
    AsyncAsyncJobsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from .upload_schedules import (
    UploadSchedulesResource,
    AsyncUploadSchedulesResource,
    UploadSchedulesResourceWithRawResponse,
    AsyncUploadSchedulesResourceWithRawResponse,
    UploadSchedulesResourceWithStreamingResponse,
    AsyncUploadSchedulesResourceWithStreamingResponse,
)
from ...types.knowledge_base import KnowledgeBase
from .data_source_connections import (
    DataSourceConnectionsResource,
    AsyncDataSourceConnectionsResource,
    DataSourceConnectionsResourceWithRawResponse,
    AsyncDataSourceConnectionsResourceWithRawResponse,
    DataSourceConnectionsResourceWithStreamingResponse,
    AsyncDataSourceConnectionsResourceWithStreamingResponse,
)
from ...types.create_knowledge_base_response import CreateKnowledgeBaseResponse
from ...types.delete_knowledge_base_response import DeleteKnowledgeBaseResponse
from ...types.create_knowledge_base_uploads_from_files_response import CreateKnowledgeBaseUploadsFromFilesResponse

__all__ = ["KnowledgeBasesResource", "AsyncKnowledgeBasesResource"]


class KnowledgeBasesResource(SyncAPIResource):
    @cached_property
    def async_jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def chunks(self) -> ChunksResource:
        return ChunksResource(self._client)

    @cached_property
    def data_source_connections(self) -> DataSourceConnectionsResource:
        return DataSourceConnectionsResource(self._client)

    @cached_property
    def uploads(self) -> UploadsResource:
        return UploadsResource(self._client)

    @cached_property
    def artifacts(self) -> ArtifactsResource:
        return ArtifactsResource(self._client)

    @cached_property
    def upload_schedules(self) -> UploadSchedulesResource:
        return UploadSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> KnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return KnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return KnowledgeBasesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        embedding_config: knowledge_base_create_params.EmbeddingConfig,
        knowledge_base_name: str,
        account_id: str | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseResponse:
        """
        ### Description

        Creates an EGP knowledge base.

        ### Details

        A knowledge base is a storage device for all data that needs to be accessible to
        EGP models. Users can upload data from a variety of data sources into a
        knowledge base, and then query the knowledge base for chunks that are
        semantically relevant to the query.

        Every knowledge base must be associated with a fixed embedding model. This
        embedding model will be used to embed all data that is stored in the knowledge
        base. The embedding model cannot be changed once the knowledge base is created.
        Only the embedding models in the dropdown menu below are supported.

        #### Differences from V1

        - V1 data ingestion consisted of knowledge bases, vector stores, and data
          connectors. V1 Knowledge bases interacted with natural language, V1 vector
          stores interacted with chunks and embeddings, and V1 data connectors set up
          automatic ingestion pipelines with third party data sources.
        - In V2, all data ingestion is done through knowledge bases. Low level
          configuration such as chunking strategies and data sources are now handled by
          this unified knowledge base v2 upload API.
        - The way data is stores in V2 allows for better observability on the ingestion
          progress and content of the knowledge base.
        - Reliability and scalability is also improved via distributed temporal
          workflows.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          embedding_config: The configuration of the embedding

          knowledge_base_name: A unique name for the knowledge base

          account_id: Account to create knowledge base in. If you have access to more than one
              account, you must specify an account_id

          metadata: Metadata associated with the knowledge base

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/knowledge-bases",
            body=maybe_transform(
                {
                    "embedding_config": embedding_config,
                    "knowledge_base_name": knowledge_base_name,
                    "account_id": account_id,
                    "metadata": metadata,
                },
                knowledge_base_create_params.KnowledgeBaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseResponse,
        )

    def retrieve(
        self,
        knowledge_base_id: str,
        *,
        include_artifacts_status: bool | NotGiven = NOT_GIVEN,
        view: List[Literal["Connections", "ArtifactCount"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBase:
        """
        ### Description

        Gets the details of a knowledge base.

        ### Details

        This API can be used to get information about a single knowledge base by ID. To
        use this API, pass in the `knowledge_base_id` that was returned from your
        [Create Knowledge Base API](https://scale-egp.readme.io/reference/create_knowledge_base_v2)
        call as a path parameter.

        This API will return the details of a knowledge base including its ID, name, the
        embedding model it uses, any metadata associated with the knowledge base, and
        the timestamps for its creation, last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          include_artifacts_status: Optional query parameter to include a count of artifacts by status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_artifacts_status": include_artifacts_status,
                        "view": view,
                    },
                    knowledge_base_retrieve_params.KnowledgeBaseRetrieveParams,
                ),
            ),
            cast_to=KnowledgeBase,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["Connections", "ArtifactCount"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[KnowledgeBase]:
        """
        ### Description

        Lists all knowledge bases owned by the authorized user.

        ### Details

        This API can be used to list all knowledge bases that have been created by the
        user. This API will return the details of all knowledge bases including their
        IDs, names, the embedding models they use, any metadata associated with the
        knowledge bases, and the timestamps for their creation, last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          account_id: Optional search by account_id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/knowledge-bases",
            page=SyncPageResponse[KnowledgeBase],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    knowledge_base_list_params.KnowledgeBaseListParams,
                ),
            ),
            model=KnowledgeBase,
        )

    def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeleteKnowledgeBaseResponse:
        """
        ### Description

        Deletes a knowledge base.

        ### Details

        This API can be used to delete a knowledge base by ID. To use this API, pass in
        the `knowledge_base_id` that was returned from your
        [Create Knowledge Base API](https://scale-egp.readme.io/reference/create_knowledge_base_v2)
        call as a path parameter.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._delete(
            f"/v4/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeleteKnowledgeBaseResponse,
        )

    def query(
        self,
        knowledge_base_id: str,
        *,
        query: str,
        top_k: int,
        include_embeddings: bool | NotGiven = NOT_GIVEN,
        metadata_filters: object | NotGiven = NOT_GIVEN,
        verbose: bool | NotGiven = NOT_GIVEN,
        wildcard_filters: Dict[str, knowledge_base_query_params.WildcardFilters] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        ### Description

        Query a knowledge base for text chunks that are most semantically relevant to
        the query.

        ### Details

        Given a query expressed as an embedding, this API runs a similarity search
        amongst the embeddings indexed in the knowledge base to find the most relevant
        chunk embeddings. To use this API, specify the `knowledge_base_id` of the
        knowledge base you want to query, pass in the natural language `query` that you
        want to search for, specify the value `top_k`, which is the number of similar
        chunks that will be returned, and specify whether you want the returned chunks
        to `include_embeddings`.

        Similarity search is used to efficiently find, retrieve, and rank chunks based
        on their similarity to a given query, which is also expressed as an embedding.
        Similarity scores ( using the cosine similarity metric) are calculated between
        each chunk embedding and the embedded query, and the chunks are ranked based on
        similarity score. The top-ranked chunks are returned as the query results.

        We are using the Hierarchical Navigable Small World (HNSW) algorithm to perform
        a k nearest neighbors search in the vector space. This algorithm returns an
        estimate of the best k nearest neighbors and is optimized for datasets with
        hundreds of thousands of vectors. You can read more about the specifics of this
        algorithm
        [here](https://opensearch.org/docs/1.0/search-plugins/knn/approximate-knn/).

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          query: The natural language query to be answered by referencing the data ingested into
              the knowledge base

          top_k: Number of chunks to return. Must be greater than 0 if specified. If not
              specified, all chunks will be returned.

          include_embeddings: Whether or not to include the embeddings for each chunk

          metadata_filters: Optional filter by metadata fields, encoded as a JSON object

          verbose: Enable or disable verbose logging

          wildcard_filters: Optional wildcard filter for fields. Only fields that are part of the metadata
              will be accepted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/query",
            body=maybe_transform(
                {
                    "query": query,
                    "top_k": top_k,
                    "include_embeddings": include_embeddings,
                    "metadata_filters": metadata_filters,
                    "verbose": verbose,
                    "wildcard_filters": wildcard_filters,
                },
                knowledge_base_query_params.KnowledgeBaseQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def upload_files(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: str,
        data_source_config: str,
        files: List[FileTypes],
        force_reupload: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadsFromFilesResponse:
        """
        Submit Upload Job with local files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        body = deepcopy_minimal(
            {
                "chunking_strategy_config": chunking_strategy_config,
                "data_source_config": data_source_config,
                "files": files,
                "force_reupload": force_reupload,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload_files",
            body=maybe_transform(body, knowledge_base_upload_files_params.KnowledgeBaseUploadFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseUploadsFromFilesResponse,
        )


class AsyncKnowledgeBasesResource(AsyncAPIResource):
    @cached_property
    def async_jobs(self) -> AsyncAsyncJobsResource:
        return AsyncAsyncJobsResource(self._client)

    @cached_property
    def chunks(self) -> AsyncChunksResource:
        return AsyncChunksResource(self._client)

    @cached_property
    def data_source_connections(self) -> AsyncDataSourceConnectionsResource:
        return AsyncDataSourceConnectionsResource(self._client)

    @cached_property
    def uploads(self) -> AsyncUploadsResource:
        return AsyncUploadsResource(self._client)

    @cached_property
    def artifacts(self) -> AsyncArtifactsResource:
        return AsyncArtifactsResource(self._client)

    @cached_property
    def upload_schedules(self) -> AsyncUploadSchedulesResource:
        return AsyncUploadSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncKnowledgeBasesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        embedding_config: knowledge_base_create_params.EmbeddingConfig,
        knowledge_base_name: str,
        account_id: str | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseResponse:
        """
        ### Description

        Creates an EGP knowledge base.

        ### Details

        A knowledge base is a storage device for all data that needs to be accessible to
        EGP models. Users can upload data from a variety of data sources into a
        knowledge base, and then query the knowledge base for chunks that are
        semantically relevant to the query.

        Every knowledge base must be associated with a fixed embedding model. This
        embedding model will be used to embed all data that is stored in the knowledge
        base. The embedding model cannot be changed once the knowledge base is created.
        Only the embedding models in the dropdown menu below are supported.

        #### Differences from V1

        - V1 data ingestion consisted of knowledge bases, vector stores, and data
          connectors. V1 Knowledge bases interacted with natural language, V1 vector
          stores interacted with chunks and embeddings, and V1 data connectors set up
          automatic ingestion pipelines with third party data sources.
        - In V2, all data ingestion is done through knowledge bases. Low level
          configuration such as chunking strategies and data sources are now handled by
          this unified knowledge base v2 upload API.
        - The way data is stores in V2 allows for better observability on the ingestion
          progress and content of the knowledge base.
        - Reliability and scalability is also improved via distributed temporal
          workflows.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          embedding_config: The configuration of the embedding

          knowledge_base_name: A unique name for the knowledge base

          account_id: Account to create knowledge base in. If you have access to more than one
              account, you must specify an account_id

          metadata: Metadata associated with the knowledge base

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/knowledge-bases",
            body=await async_maybe_transform(
                {
                    "embedding_config": embedding_config,
                    "knowledge_base_name": knowledge_base_name,
                    "account_id": account_id,
                    "metadata": metadata,
                },
                knowledge_base_create_params.KnowledgeBaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseResponse,
        )

    async def retrieve(
        self,
        knowledge_base_id: str,
        *,
        include_artifacts_status: bool | NotGiven = NOT_GIVEN,
        view: List[Literal["Connections", "ArtifactCount"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBase:
        """
        ### Description

        Gets the details of a knowledge base.

        ### Details

        This API can be used to get information about a single knowledge base by ID. To
        use this API, pass in the `knowledge_base_id` that was returned from your
        [Create Knowledge Base API](https://scale-egp.readme.io/reference/create_knowledge_base_v2)
        call as a path parameter.

        This API will return the details of a knowledge base including its ID, name, the
        embedding model it uses, any metadata associated with the knowledge base, and
        the timestamps for its creation, last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          include_artifacts_status: Optional query parameter to include a count of artifacts by status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_artifacts_status": include_artifacts_status,
                        "view": view,
                    },
                    knowledge_base_retrieve_params.KnowledgeBaseRetrieveParams,
                ),
            ),
            cast_to=KnowledgeBase,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["Connections", "ArtifactCount"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[KnowledgeBase, AsyncPageResponse[KnowledgeBase]]:
        """
        ### Description

        Lists all knowledge bases owned by the authorized user.

        ### Details

        This API can be used to list all knowledge bases that have been created by the
        user. This API will return the details of all knowledge bases including their
        IDs, names, the embedding models they use, any metadata associated with the
        knowledge bases, and the timestamps for their creation, last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          account_id: Optional search by account_id

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/knowledge-bases",
            page=AsyncPageResponse[KnowledgeBase],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    knowledge_base_list_params.KnowledgeBaseListParams,
                ),
            ),
            model=KnowledgeBase,
        )

    async def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeleteKnowledgeBaseResponse:
        """
        ### Description

        Deletes a knowledge base.

        ### Details

        This API can be used to delete a knowledge base by ID. To use this API, pass in
        the `knowledge_base_id` that was returned from your
        [Create Knowledge Base API](https://scale-egp.readme.io/reference/create_knowledge_base_v2)
        call as a path parameter.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._delete(
            f"/v4/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeleteKnowledgeBaseResponse,
        )

    async def query(
        self,
        knowledge_base_id: str,
        *,
        query: str,
        top_k: int,
        include_embeddings: bool | NotGiven = NOT_GIVEN,
        metadata_filters: object | NotGiven = NOT_GIVEN,
        verbose: bool | NotGiven = NOT_GIVEN,
        wildcard_filters: Dict[str, knowledge_base_query_params.WildcardFilters] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        ### Description

        Query a knowledge base for text chunks that are most semantically relevant to
        the query.

        ### Details

        Given a query expressed as an embedding, this API runs a similarity search
        amongst the embeddings indexed in the knowledge base to find the most relevant
        chunk embeddings. To use this API, specify the `knowledge_base_id` of the
        knowledge base you want to query, pass in the natural language `query` that you
        want to search for, specify the value `top_k`, which is the number of similar
        chunks that will be returned, and specify whether you want the returned chunks
        to `include_embeddings`.

        Similarity search is used to efficiently find, retrieve, and rank chunks based
        on their similarity to a given query, which is also expressed as an embedding.
        Similarity scores ( using the cosine similarity metric) are calculated between
        each chunk embedding and the embedded query, and the chunks are ranked based on
        similarity score. The top-ranked chunks are returned as the query results.

        We are using the Hierarchical Navigable Small World (HNSW) algorithm to perform
        a k nearest neighbors search in the vector space. This algorithm returns an
        estimate of the best k nearest neighbors and is optimized for datasets with
        hundreds of thousands of vectors. You can read more about the specifics of this
        algorithm
        [here](https://opensearch.org/docs/1.0/search-plugins/knn/approximate-knn/).

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          query: The natural language query to be answered by referencing the data ingested into
              the knowledge base

          top_k: Number of chunks to return. Must be greater than 0 if specified. If not
              specified, all chunks will be returned.

          include_embeddings: Whether or not to include the embeddings for each chunk

          metadata_filters: Optional filter by metadata fields, encoded as a JSON object

          verbose: Enable or disable verbose logging

          wildcard_filters: Optional wildcard filter for fields. Only fields that are part of the metadata
              will be accepted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/query",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "top_k": top_k,
                    "include_embeddings": include_embeddings,
                    "metadata_filters": metadata_filters,
                    "verbose": verbose,
                    "wildcard_filters": wildcard_filters,
                },
                knowledge_base_query_params.KnowledgeBaseQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def upload_files(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: str,
        data_source_config: str,
        files: List[FileTypes],
        force_reupload: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadsFromFilesResponse:
        """
        Submit Upload Job with local files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        body = deepcopy_minimal(
            {
                "chunking_strategy_config": chunking_strategy_config,
                "data_source_config": data_source_config,
                "files": files,
                "force_reupload": force_reupload,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload_files",
            body=await async_maybe_transform(body, knowledge_base_upload_files_params.KnowledgeBaseUploadFilesParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseUploadsFromFilesResponse,
        )


class KnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = to_raw_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = to_raw_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.list = to_raw_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = to_raw_response_wrapper(
            knowledge_bases.delete,
        )
        self.query = to_raw_response_wrapper(
            knowledge_bases.query,
        )
        self.upload_files = to_raw_response_wrapper(
            knowledge_bases.upload_files,
        )

    @cached_property
    def async_jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._knowledge_bases.async_jobs)

    @cached_property
    def chunks(self) -> ChunksResourceWithRawResponse:
        return ChunksResourceWithRawResponse(self._knowledge_bases.chunks)

    @cached_property
    def data_source_connections(self) -> DataSourceConnectionsResourceWithRawResponse:
        return DataSourceConnectionsResourceWithRawResponse(self._knowledge_bases.data_source_connections)

    @cached_property
    def uploads(self) -> UploadsResourceWithRawResponse:
        return UploadsResourceWithRawResponse(self._knowledge_bases.uploads)

    @cached_property
    def artifacts(self) -> ArtifactsResourceWithRawResponse:
        return ArtifactsResourceWithRawResponse(self._knowledge_bases.artifacts)

    @cached_property
    def upload_schedules(self) -> UploadSchedulesResourceWithRawResponse:
        return UploadSchedulesResourceWithRawResponse(self._knowledge_bases.upload_schedules)


class AsyncKnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = async_to_raw_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = async_to_raw_response_wrapper(
            knowledge_bases.delete,
        )
        self.query = async_to_raw_response_wrapper(
            knowledge_bases.query,
        )
        self.upload_files = async_to_raw_response_wrapper(
            knowledge_bases.upload_files,
        )

    @cached_property
    def async_jobs(self) -> AsyncAsyncJobsResourceWithRawResponse:
        return AsyncAsyncJobsResourceWithRawResponse(self._knowledge_bases.async_jobs)

    @cached_property
    def chunks(self) -> AsyncChunksResourceWithRawResponse:
        return AsyncChunksResourceWithRawResponse(self._knowledge_bases.chunks)

    @cached_property
    def data_source_connections(self) -> AsyncDataSourceConnectionsResourceWithRawResponse:
        return AsyncDataSourceConnectionsResourceWithRawResponse(self._knowledge_bases.data_source_connections)

    @cached_property
    def uploads(self) -> AsyncUploadsResourceWithRawResponse:
        return AsyncUploadsResourceWithRawResponse(self._knowledge_bases.uploads)

    @cached_property
    def artifacts(self) -> AsyncArtifactsResourceWithRawResponse:
        return AsyncArtifactsResourceWithRawResponse(self._knowledge_bases.artifacts)

    @cached_property
    def upload_schedules(self) -> AsyncUploadSchedulesResourceWithRawResponse:
        return AsyncUploadSchedulesResourceWithRawResponse(self._knowledge_bases.upload_schedules)


class KnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = to_streamed_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = to_streamed_response_wrapper(
            knowledge_bases.delete,
        )
        self.query = to_streamed_response_wrapper(
            knowledge_bases.query,
        )
        self.upload_files = to_streamed_response_wrapper(
            knowledge_bases.upload_files,
        )

    @cached_property
    def async_jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._knowledge_bases.async_jobs)

    @cached_property
    def chunks(self) -> ChunksResourceWithStreamingResponse:
        return ChunksResourceWithStreamingResponse(self._knowledge_bases.chunks)

    @cached_property
    def data_source_connections(self) -> DataSourceConnectionsResourceWithStreamingResponse:
        return DataSourceConnectionsResourceWithStreamingResponse(self._knowledge_bases.data_source_connections)

    @cached_property
    def uploads(self) -> UploadsResourceWithStreamingResponse:
        return UploadsResourceWithStreamingResponse(self._knowledge_bases.uploads)

    @cached_property
    def artifacts(self) -> ArtifactsResourceWithStreamingResponse:
        return ArtifactsResourceWithStreamingResponse(self._knowledge_bases.artifacts)

    @cached_property
    def upload_schedules(self) -> UploadSchedulesResourceWithStreamingResponse:
        return UploadSchedulesResourceWithStreamingResponse(self._knowledge_bases.upload_schedules)


class AsyncKnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = async_to_streamed_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            knowledge_bases.delete,
        )
        self.query = async_to_streamed_response_wrapper(
            knowledge_bases.query,
        )
        self.upload_files = async_to_streamed_response_wrapper(
            knowledge_bases.upload_files,
        )

    @cached_property
    def async_jobs(self) -> AsyncAsyncJobsResourceWithStreamingResponse:
        return AsyncAsyncJobsResourceWithStreamingResponse(self._knowledge_bases.async_jobs)

    @cached_property
    def chunks(self) -> AsyncChunksResourceWithStreamingResponse:
        return AsyncChunksResourceWithStreamingResponse(self._knowledge_bases.chunks)

    @cached_property
    def data_source_connections(self) -> AsyncDataSourceConnectionsResourceWithStreamingResponse:
        return AsyncDataSourceConnectionsResourceWithStreamingResponse(self._knowledge_bases.data_source_connections)

    @cached_property
    def uploads(self) -> AsyncUploadsResourceWithStreamingResponse:
        return AsyncUploadsResourceWithStreamingResponse(self._knowledge_bases.uploads)

    @cached_property
    def artifacts(self) -> AsyncArtifactsResourceWithStreamingResponse:
        return AsyncArtifactsResourceWithStreamingResponse(self._knowledge_bases.artifacts)

    @cached_property
    def upload_schedules(self) -> AsyncUploadSchedulesResourceWithStreamingResponse:
        return AsyncUploadSchedulesResourceWithStreamingResponse(self._knowledge_bases.upload_schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ChunkRankParams",
    "RankStrategy",
    "RankStrategyCrossEncoderRankStrategy",
    "RankStrategyCrossEncoderRankStrategyParams",
    "RankStrategyRougeRankStrategy",
    "RankStrategyRougeRankStrategyParams",
    "RankStrategyModelRankStrategy",
    "RankStrategyModelRankStrategyParams",
    "RelevantChunk",
]


class ChunkRankParams(TypedDict, total=False):
    query: Required[str]
    """Natural language query to re-rank chunks against.

    If a vector store query was originally used to retrieve these chunks, please use
    the same query for this ranking
    """

    rank_strategy: Required[RankStrategy]
    """The ranking strategy to use.

    Rank strategies determine how the ranking is done, They consist of the ranking
    method name and additional params needed to compute the ranking.

    Use the built-in `cross_encoder` or `rouge` strategies or create a custom one
    with the Models API.
    """

    relevant_chunks: Required[Iterable[RelevantChunk]]
    """List of chunks to rank"""

    account_id: str
    """Account to rank chunks with.

    If you have access to more than one account, you must specify an account_id
    """

    top_k: int
    """Number of chunks to return.

    Must be greater than 0 if specified. If not specified, all chunks will be
    returned.
    """


class RankStrategyCrossEncoderRankStrategyParams(TypedDict, total=False):
    cross_encoder_model: Literal["cross-encoder/ms-marco-MiniLM-L-12-v2", "cross-encoder/mmarco-mMiniLMv2-L12-H384-v1"]
    """Cross encoder model to use when ranking.

    Currently supports
    [cross-encoder/ms-marco-MiniLM-L-12-v2](https://huggingface.co/cross-encoder/ms-marco-MiniLM-L-12-v2)
    and
    [cross-encoder/mmarco-mMiniLMv2-L12-H384-v1](https://huggingface.co/cross-encoder/mmarco-mMiniLMv2-L12-H384-v1).
    """


class RankStrategyCrossEncoderRankStrategy(TypedDict, total=False):
    method: Literal["cross_encoder"]
    """The name of the rank strategy. Must be `cross_encoder`."""

    params: RankStrategyCrossEncoderRankStrategyParams
    """The parameters needed for ranking."""


class RankStrategyRougeRankStrategyParams(TypedDict, total=False):
    metric: str
    """Rouge type, can be n-gram based (e.g.

    rouge1, rouge2) or longest common subsequence (rougeL or rougeLsum)
    """

    score: Literal["precision", "recall", "fmeasure"]
    """Metric to use from Rouge score"""


class RankStrategyRougeRankStrategy(TypedDict, total=False):
    method: Literal["rouge"]
    """The name of the rank strategy."""

    params: RankStrategyRougeRankStrategyParams
    """The parameters needed for ranking."""


class RankStrategyModelRankStrategyParams(TypedDict, total=False):
    base_model_name: str
    """The name of a base model to use for reranking"""

    model_deployment_id: str
    """The model deployment id of a custom model to use for reranking"""

    model_params: object


class RankStrategyModelRankStrategy(TypedDict, total=False):
    method: Literal["model"]
    """Use a model from Models API for ranking."""

    params: RankStrategyModelRankStrategyParams
    """The parameters needed for ranking."""


RankStrategy: TypeAlias = Union[
    RankStrategyCrossEncoderRankStrategy, RankStrategyRougeRankStrategy, RankStrategyModelRankStrategy
]


class RelevantChunk(TypedDict, total=False):
    chunk_id: Required[str]
    """The unique ID of the chunk with embedding"""

    score: Required[float]
    """
    A number between 0 and 1 representing how similar a chunk's embedding is to the
    query embedding. Higher numbers mean that this chunk with embedding is more
    similar.
    """

    text: Required[str]
    """The text associated with the chunk"""

    attachment_url: str
    """Original attachment URL from which this chunk got its data from"""

    embedding: Iterable[float]
    """The vector embedding of the text associated with the chunk"""

    metadata: object
    """
    Any additional key value pairs of information stored by you on the chunk with
    embedding
    """

    title: str
    """Title for this chunk, for example the file name"""

    user_supplied_metadata: object
    """
    Any additional key value pairs of information returned from the custom chunking.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["SummarizationAnnotationConfigParam"]


class SummarizationAnnotationConfigParam(TypedDict, total=False):
    document_loc: Required[List[str]]

    summary_loc: Required[List[str]]

    expected_summary_loc: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..token_chunking_strategy_config import TokenChunkingStrategyConfig
from ..custom_chunking_strategy_config import CustomChunkingStrategyConfig
from ..character_chunking_strategy_config import CharacterChunkingStrategyConfig

__all__ = ["UploadSchedule", "ChunkingStrategyConfig"]

ChunkingStrategyConfig: TypeAlias = Annotated[
    Union[CharacterChunkingStrategyConfig, TokenChunkingStrategyConfig, CustomChunkingStrategyConfig],
    PropertyInfo(discriminator="strategy"),
]


class UploadSchedule(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    chunking_strategy_config: ChunkingStrategyConfig

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    interval: float

    knowledge_base_data_source_id: str

    knowledge_base_id: str

    status: Literal["HEALTHY", "UNHEALTHY", "ERROR", "PAUSED"]
    """An enumeration."""

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    next_run_at: Optional[datetime] = None

    status_reason: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, TypedDict

__all__ = ["ParameterBindingsParam"]


class ParameterBindingsParam(TypedDict, total=False):
    bindings: Required[Dict[str, Union[str, float, bool]]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chunk_extra_info_schema import ChunkExtraInfoSchema as ChunkExtraInfoSchema
from .result_schema_generation import ResultSchemaGeneration as ResultSchemaGeneration
from .string_extra_info_schema import StringExtraInfoSchema as StringExtraInfoSchema



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ApplicationNode", "Configuration"]


class Configuration(BaseModel):
    value: object


class ApplicationNode(BaseModel):
    id: str

    application_node_schema_id: Literal[
        "text_input_schema",
        "text_output_schema",
        "knowledge_base_schema",
        "reranker_schema",
        "prompt_engineering_schema",
        "completion_model_schema",
        "external_endpoint_schema",
        "document_input_schema",
        "map_reduce_schema",
        "document_search_schema",
        "document_prompt_schema",
    ]
    """An enumeration."""

    configuration: Optional[Dict[str, Configuration]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .test_case import TestCase

__all__ = ["TestCaseBatchResponse"]

TestCaseBatchResponse: TypeAlias = List[TestCase]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import chat_thread_update_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .messages.messages import MessagesResource, AsyncMessagesResource
from ...types.chat_thread import ChatThread

__all__ = ["ChatThreadsResource", "AsyncChatThreadsResource"]


class ChatThreadsResource(SyncAPIResource):
    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ChatThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChatThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChatThreadsResourceWithStreamingResponse(self)

    def update(
        self,
        thread_id: str,
        *,
        archived_at: str | NotGiven = NOT_GIVEN,
        title: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatThread:
        """
        Patch Chat Thread

        Args:
          archived_at: Date when the chat thread is archived, or None to un-archive.

          title: The title of the chat thread.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return self._patch(
            f"/v4/threads/{thread_id}",
            body=maybe_transform(
                {
                    "archived_at": archived_at,
                    "title": title,
                },
                chat_thread_update_params.ChatThreadUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatThread,
        )

    def delete(
        self,
        thread_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete Chat Thread

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return self._delete(
            f"/v4/threads/{thread_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncChatThreadsResource(AsyncAPIResource):
    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncChatThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChatThreadsResourceWithStreamingResponse(self)

    async def update(
        self,
        thread_id: str,
        *,
        archived_at: str | NotGiven = NOT_GIVEN,
        title: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatThread:
        """
        Patch Chat Thread

        Args:
          archived_at: Date when the chat thread is archived, or None to un-archive.

          title: The title of the chat thread.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return await self._patch(
            f"/v4/threads/{thread_id}",
            body=await async_maybe_transform(
                {
                    "archived_at": archived_at,
                    "title": title,
                },
                chat_thread_update_params.ChatThreadUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatThread,
        )

    async def delete(
        self,
        thread_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete Chat Thread

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return await self._delete(
            f"/v4/threads/{thread_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ChatThreadsResourceWithRawResponse:
    def __init__(self, chat_threads: ChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.update = to_raw_response_wrapper(
            chat_threads.update,
        )
        self.delete = to_raw_response_wrapper(
            chat_threads.delete,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._chat_threads.messages)


class AsyncChatThreadsResourceWithRawResponse:
    def __init__(self, chat_threads: AsyncChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.update = async_to_raw_response_wrapper(
            chat_threads.update,
        )
        self.delete = async_to_raw_response_wrapper(
            chat_threads.delete,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._chat_threads.messages)


class ChatThreadsResourceWithStreamingResponse:
    def __init__(self, chat_threads: ChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.update = to_streamed_response_wrapper(
            chat_threads.update,
        )
        self.delete = to_streamed_response_wrapper(
            chat_threads.delete,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._chat_threads.messages)


class AsyncChatThreadsResourceWithStreamingResponse:
    def __init__(self, chat_threads: AsyncChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.update = async_to_streamed_response_wrapper(
            chat_threads.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            chat_threads.delete,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._chat_threads.messages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .evaluation_trace_span import EvaluationTraceSpan
from .application_metric_score import ApplicationMetricScore
from .evaluation_datasets.test_case import TestCase
from .shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from .shared.result_schema_generation import ResultSchemaGeneration
from .shared.string_extra_info_schema import StringExtraInfoSchema
from .evaluation_datasets.flexible_chunk import FlexibleChunk
from .evaluation_datasets.flexible_message import FlexibleMessage

__all__ = [
    "ApplicationTestCaseOutputListResponse",
    "ApplicationTestCaseGenerationOutputResponseWithViews",
    "ApplicationTestCaseGenerationOutputResponseWithViewsInteraction",
    "ApplicationTestCaseFlexibleOutputResponseWithViews",
    "ApplicationTestCaseFlexibleOutputResponseWithViewsOutput",
    "ApplicationTestCaseFlexibleOutputResponseWithViewsOutputGenerationExtraInfo",
    "ApplicationTestCaseFlexibleOutputResponseWithViewsInteraction",
]


class ApplicationTestCaseGenerationOutputResponseWithViewsInteraction(BaseModel):
    id: str

    aggregated: bool

    application_spec_id: str

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    duration_ms: int
    """How much time the step took in milliseconds(ms)"""

    input: object

    operation_status: Literal["SUCCESS", "ERROR"]
    """An enumeration."""

    output: object

    start_timestamp: datetime

    chat_thread_id: Optional[str] = None

    interaction_source: Optional[Literal["EXTERNAL_AI", "EVALUATION", "SGP_CHAT"]] = None
    """An enumeration."""

    operation_metadata: Optional[object] = None
    """The JSON representation of the metadata insights emitted through the execution.

    This can differ based on different types of operations
    """

    trace_spans: Optional[List[EvaluationTraceSpan]] = None


class ApplicationTestCaseGenerationOutputResponseWithViews(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    evaluation_dataset_id: str

    output: ResultSchemaGeneration

    test_case_id: str

    application_interaction_id: Optional[str] = None

    interaction: Optional[ApplicationTestCaseGenerationOutputResponseWithViewsInteraction] = None

    metric_scores: Optional[List[ApplicationMetricScore]] = None

    metrics: Optional[Dict[str, float]] = None

    schema_type: Optional[Literal["GENERATION"]] = None

    test_case_version: Optional[TestCase] = None


ApplicationTestCaseFlexibleOutputResponseWithViewsOutputGenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class ApplicationTestCaseFlexibleOutputResponseWithViewsOutput(BaseModel):
    generation_output: Union[
        str,
        float,
        List[FlexibleChunk],
        List[FlexibleMessage],
        List[object],
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]],
        object,
    ]

    generation_extra_info: Optional[ApplicationTestCaseFlexibleOutputResponseWithViewsOutputGenerationExtraInfo] = None


class ApplicationTestCaseFlexibleOutputResponseWithViewsInteraction(BaseModel):
    id: str

    aggregated: bool

    application_spec_id: str

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    duration_ms: int
    """How much time the step took in milliseconds(ms)"""

    input: object

    operation_status: Literal["SUCCESS", "ERROR"]
    """An enumeration."""

    output: object

    start_timestamp: datetime

    chat_thread_id: Optional[str] = None

    interaction_source: Optional[Literal["EXTERNAL_AI", "EVALUATION", "SGP_CHAT"]] = None
    """An enumeration."""

    operation_metadata: Optional[object] = None
    """The JSON representation of the metadata insights emitted through the execution.

    This can differ based on different types of operations
    """

    trace_spans: Optional[List[EvaluationTraceSpan]] = None


class ApplicationTestCaseFlexibleOutputResponseWithViews(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    evaluation_dataset_id: str

    output: ApplicationTestCaseFlexibleOutputResponseWithViewsOutput

    test_case_id: str

    application_interaction_id: Optional[str] = None

    interaction: Optional[ApplicationTestCaseFlexibleOutputResponseWithViewsInteraction] = None

    metric_scores: Optional[List[ApplicationMetricScore]] = None

    metrics: Optional[Dict[str, float]] = None

    schema_type: Optional[Literal["FLEXIBLE"]] = None

    test_case_version: Optional[TestCase] = None


ApplicationTestCaseOutputListResponse: TypeAlias = Annotated[
    Union[ApplicationTestCaseGenerationOutputResponseWithViews, ApplicationTestCaseFlexibleOutputResponseWithViews],
    PropertyInfo(discriminator="schema_type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPageResponse, AsyncPageResponse
from ...._base_client import AsyncPaginator, make_request_options
from ....types.evaluations.test_case_result import TestCaseResult
from ....types.evaluations.test_case_results import history_list_params

__all__ = ["HistoryResource", "AsyncHistoryResource"]


class HistoryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return HistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return HistoryResourceWithStreamingResponse(self)

    def retrieve(
        self,
        num: str,
        *,
        evaluation_id: str,
        test_case_result_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        Get Historical Test Case Result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        if not num:
            raise ValueError(f"Expected a non-empty value for `num` but received {num!r}")
        return cast(
            TestCaseResult,
            self._get(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}/history/{num}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        num: str,
        *,
        evaluation_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[TestCaseResult]:
        """
        List Historical Test Case Results

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not num:
            raise ValueError(f"Expected a non-empty value for `num` but received {num!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/test-case-results/history/{num}",
            page=SyncPageResponse[TestCaseResult],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    history_list_params.HistoryListParams,
                ),
            ),
            model=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncHistoryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncHistoryResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        num: str,
        *,
        evaluation_id: str,
        test_case_result_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        Get Historical Test Case Result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        if not num:
            raise ValueError(f"Expected a non-empty value for `num` but received {num!r}")
        return cast(
            TestCaseResult,
            await self._get(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}/history/{num}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        num: str,
        *,
        evaluation_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TestCaseResult, AsyncPageResponse[TestCaseResult]]:
        """
        List Historical Test Case Results

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not num:
            raise ValueError(f"Expected a non-empty value for `num` but received {num!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/test-case-results/history/{num}",
            page=AsyncPageResponse[TestCaseResult],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    history_list_params.HistoryListParams,
                ),
            ),
            model=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
        )


class HistoryResourceWithRawResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

        self.retrieve = to_raw_response_wrapper(
            history.retrieve,
        )
        self.list = to_raw_response_wrapper(
            history.list,
        )


class AsyncHistoryResourceWithRawResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

        self.retrieve = async_to_raw_response_wrapper(
            history.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            history.list,
        )


class HistoryResourceWithStreamingResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

        self.retrieve = to_streamed_response_wrapper(
            history.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            history.list,
        )


class AsyncHistoryResourceWithStreamingResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

        self.retrieve = async_to_streamed_response_wrapper(
            history.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            history.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["ApplicationVariantReportCreateParams"]


class ApplicationVariantReportCreateParams(TypedDict, total=False):
    application_variant_id: Required[str]

    evaluation_dataset_ids: Required[List[str]]

    account_id: str
    """The ID of the account that owns the given entity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["DeleteKnowledgeBaseResponse"]


class DeleteKnowledgeBaseResponse(BaseModel):
    deleted: bool
    """Whether or not the knowledge base was successfully deleted"""

    knowledge_base_id: str
    """The ID of the knowledge base that was deleted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AzureBlobStorageDataSourceConfig"]


class AzureBlobStorageDataSourceConfig(BaseModel):
    container_url: str
    """
    The full URL of the container such as
    'https://your-account-name.blob.core.windows.net/your-container-name'
    """

    source: Literal["AzureBlobStorage"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .evaluation_datasets.flexible_chunk import FlexibleChunk
from .evaluation_datasets.flexible_message import FlexibleMessage

__all__ = ["EvaluationTraceSpan"]


class EvaluationTraceSpan(BaseModel):
    id: str
    """Identifies the application step"""

    application_interaction_id: str
    """The id of the application insight this step belongs to"""

    duration_ms: int
    """How much time the step took in milliseconds(ms)"""

    node_id: str
    """The id of the node in the application_variant config that emitted this insight"""

    operation_input: Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]]
    """The JSON representation of the input that this step received."""

    operation_output: Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]]
    """The JSON representation of the output that this step emitted."""

    operation_status: Literal["SUCCESS", "ERROR"]
    """An enumeration."""

    operation_type: Literal[
        "TEXT_INPUT",
        "TEXT_OUTPUT",
        "COMPLETION",
        "KB_RETRIEVAL",
        "RERANKING",
        "EXTERNAL_ENDPOINT",
        "PROMPT_ENGINEERING",
        "DOCUMENT_INPUT",
        "MAP_REDUCE",
        "DOCUMENT_SEARCH",
        "DOCUMENT_PROMPT",
        "CUSTOM",
        "INPUT_GUARDRAIL",
        "OUTPUT_GUARDRAIL",
    ]
    """An enumeration."""

    start_timestamp: datetime
    """The start time of the step"""

    operation_expected: Optional[
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]]
    ] = None
    """The JSON representation of the expected output for this step"""

    operation_metadata: Optional[object] = None
    """The JSON representation of the metadata insights emitted through the execution.

    This can differ based on different types of operations
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .application_edge_param import ApplicationEdgeParam
from .application_node_param import ApplicationNodeParam

__all__ = ["ApplicationConfigurationParam"]


class ApplicationConfigurationParam(TypedDict, total=False):
    edges: Required[Iterable[ApplicationEdgeParam]]

    nodes: Required[Iterable[ApplicationNodeParam]]

    metadata: object
    """User defined metadata about the application"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, TypedDict

__all__ = ["ModelListParams"]


class ModelListParams(TypedDict, total=False):
    account_id: str

    limit: int
    """Maximum number of artifacts to be returned by the given endpoint.

    Defaults to 100 and cannot be greater than 10k.
    """

    model_group_id: Union[int, str]

    model_type: Union[int, str]

    page: int
    """Page number for pagination to be returned by the given endpoint.

    Starts at page 1
    """

    sort_by: List[
        Literal[
            "model_vendor:asc",
            "model_vendor:desc",
            "base_model_id:asc",
            "base_model_id:desc",
            "base_model_metadata:asc",
            "base_model_metadata:desc",
            "model_creation_parameters:asc",
            "model_creation_parameters:desc",
            "model_card:asc",
            "model_card:desc",
            "training_data_card:asc",
            "training_data_card:desc",
            "description:asc",
            "description:desc",
            "model_template_id:asc",
            "model_template_id:desc",
            "model_group_id:asc",
            "model_group_id:desc",
            "model_group:asc",
            "model_group:desc",
            "request_schema:asc",
            "request_schema:desc",
            "response_schema:asc",
            "response_schema:desc",
            "deployment_count:asc",
            "deployment_count:desc",
            "supports_multi_turn:asc",
            "supports_multi_turn:desc",
            "deployments:asc",
            "deployments:desc",
            "id:asc",
            "id:desc",
            "created_at:asc",
            "created_at:desc",
            "account_id:asc",
            "account_id:desc",
            "created_by_user_id:asc",
            "created_by_user_id:desc",
            "created_by_user:asc",
            "created_by_user:desc",
            "name:asc",
            "name:desc",
            "model_type:asc",
            "model_type:desc",
        ]
    ]

    view: List[Literal["Deployments", "ModelGroup"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.timeseries_data import TimeseriesData
from ....types.applications.metrics import timesery_retrieve_params

__all__ = ["TimeseriesResource", "AsyncTimeseriesResource"]


class TimeseriesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TimeseriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return TimeseriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TimeseriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return TimeseriesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        metric_id: Literal[
            "total_requests",
            "total_errors",
            "total_tokens",
            "average_latency",
            "p95_latency",
            "error_rate",
            "average_faithfulness",
            "average_relevance",
            "average_users",
            "aggregated_tokens",
            "feedback",
            "engagement_faithfulness",
            "engagement_relevance",
            "execution_input_response_tokens",
            "execution_average_latency_per_variant",
            "execution_error_rate",
            "execution_latency_percentile",
            "execution_average_latency_per_node",
        ],
        *,
        application_spec_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        from_ts: int | NotGiven = NOT_GIVEN,
        to_ts: int | NotGiven = NOT_GIVEN,
        variants: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeseriesData:
        """
        Get Timeseries Metric

        Args:
          metric_id: An enumeration.

          account_id: Account ID used for authorization

          from_ts: The starting (oldest) timestamp window in seconds.

          to_ts: The ending (most recent) timestamp in seconds.

          variants: Which variants to filter on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_spec_id:
            raise ValueError(
                f"Expected a non-empty value for `application_spec_id` but received {application_spec_id!r}"
            )
        if not metric_id:
            raise ValueError(f"Expected a non-empty value for `metric_id` but received {metric_id!r}")
        return self._get(
            f"/v4/applications/{application_spec_id}/metrics/timeseries/{metric_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "from_ts": from_ts,
                        "to_ts": to_ts,
                        "variants": variants,
                    },
                    timesery_retrieve_params.TimeseryRetrieveParams,
                ),
            ),
            cast_to=TimeseriesData,
        )


class AsyncTimeseriesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTimeseriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTimeseriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTimeseriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncTimeseriesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        metric_id: Literal[
            "total_requests",
            "total_errors",
            "total_tokens",
            "average_latency",
            "p95_latency",
            "error_rate",
            "average_faithfulness",
            "average_relevance",
            "average_users",
            "aggregated_tokens",
            "feedback",
            "engagement_faithfulness",
            "engagement_relevance",
            "execution_input_response_tokens",
            "execution_average_latency_per_variant",
            "execution_error_rate",
            "execution_latency_percentile",
            "execution_average_latency_per_node",
        ],
        *,
        application_spec_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        from_ts: int | NotGiven = NOT_GIVEN,
        to_ts: int | NotGiven = NOT_GIVEN,
        variants: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeseriesData:
        """
        Get Timeseries Metric

        Args:
          metric_id: An enumeration.

          account_id: Account ID used for authorization

          from_ts: The starting (oldest) timestamp window in seconds.

          to_ts: The ending (most recent) timestamp in seconds.

          variants: Which variants to filter on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_spec_id:
            raise ValueError(
                f"Expected a non-empty value for `application_spec_id` but received {application_spec_id!r}"
            )
        if not metric_id:
            raise ValueError(f"Expected a non-empty value for `metric_id` but received {metric_id!r}")
        return await self._get(
            f"/v4/applications/{application_spec_id}/metrics/timeseries/{metric_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "account_id": account_id,
                        "from_ts": from_ts,
                        "to_ts": to_ts,
                        "variants": variants,
                    },
                    timesery_retrieve_params.TimeseryRetrieveParams,
                ),
            ),
            cast_to=TimeseriesData,
        )


class TimeseriesResourceWithRawResponse:
    def __init__(self, timeseries: TimeseriesResource) -> None:
        self._timeseries = timeseries

        self.retrieve = to_raw_response_wrapper(
            timeseries.retrieve,
        )


class AsyncTimeseriesResourceWithRawResponse:
    def __init__(self, timeseries: AsyncTimeseriesResource) -> None:
        self._timeseries = timeseries

        self.retrieve = async_to_raw_response_wrapper(
            timeseries.retrieve,
        )


class TimeseriesResourceWithStreamingResponse:
    def __init__(self, timeseries: TimeseriesResource) -> None:
        self._timeseries = timeseries

        self.retrieve = to_streamed_response_wrapper(
            timeseries.retrieve,
        )


class AsyncTimeseriesResourceWithStreamingResponse:
    def __init__(self, timeseries: AsyncTimeseriesResource) -> None:
        self._timeseries = timeseries

        self.retrieve = async_to_streamed_response_wrapper(
            timeseries.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SlackDataSourceConfigParam"]


class SlackDataSourceConfigParam(TypedDict, total=False):
    channel_id: Required[str]
    """Slack Channel or Conversation ID to retrieve history from.

    Open channel details and find the ID at bottom of 'About' section.
    """

    source: Required[Literal["Slack"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .evaluations import test_case_result
from .application_spec import ApplicationSpec
from .annotation_config import AnnotationConfig
from .evaluation_dataset import EvaluationDataset
from .question_set_with_questions import QuestionSetWithQuestions
from .evaluation_datasets.test_case import TestCase
from .shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from .shared.result_schema_generation import ResultSchemaGeneration
from .shared.string_extra_info_schema import StringExtraInfoSchema
from .evaluation_datasets.flexible_chunk import FlexibleChunk
from .evaluation_datasets.flexible_message import FlexibleMessage

__all__ = [
    "EvaluationWithViews",
    "AsyncJob",
    "EvaluationConfigExpanded",
    "EvaluationConfigExpandedEvaluationConfigExpanded",
    "EvaluationConfigExpandedLegacyEvaluationConfigExpanded",
    "EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSet",
    "EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSetQuestion",
    "TestCaseResult",
    "TestCaseResultGenerationTestCaseResultResponseExpanded",
    "TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResult",
    "TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata",
    "TestCaseResultGenerationTestCaseResultResponseExpandedTask",
    "TestCaseResultGenerationTestCaseResultResponseExpandedTaskAssignedTo",
    "TestCaseResultFlexibleTestCaseResultResponseExpanded",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationData",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationDataGenerationExtraInfo",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResult",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedTask",
    "TestCaseResultFlexibleTestCaseResultResponseExpandedTaskAssignedTo",
]


class AsyncJob(BaseModel):
    id: str
    """The unique identifier of the entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    status: str

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    job_metadata: Optional[object] = None

    job_type: Optional[str] = None

    parent_job_id: Optional[str] = None

    progress: Optional[object] = None


class EvaluationConfigExpandedEvaluationConfigExpanded(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_type: Literal["studio", "llm_auto", "human", "llm_benchmark"]
    """An enumeration."""

    question_set: QuestionSetWithQuestions

    question_set_id: str

    studio_project_id: Optional[str] = None


class EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSetQuestion(BaseModel):
    id: str
    """The unique identifier of the entity."""

    prompt: str

    title: str

    type: Literal["categorical", "free_text"]
    """An enumeration."""

    choices: Optional[List[object]] = None
    """List of choices for the question. Required for CATEGORICAL questions."""

    conditions: Optional[List[object]] = None
    """Conditions for the question to be shown."""

    multi: Optional[bool] = None
    """Whether the question allows multiple answers."""

    required: Optional[bool] = None
    """Whether the question is required."""


class EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSet(BaseModel):
    questions: List[EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSetQuestion]


class EvaluationConfigExpandedLegacyEvaluationConfigExpanded(BaseModel):
    evaluation_type: Literal["studio", "llm_auto", "human", "llm_benchmark"]
    """An enumeration."""

    question_set: EvaluationConfigExpandedLegacyEvaluationConfigExpandedQuestionSet

    studio_project_id: Optional[str] = None


EvaluationConfigExpanded: TypeAlias = Union[
    EvaluationConfigExpandedEvaluationConfigExpanded, EvaluationConfigExpandedLegacyEvaluationConfigExpanded
]


class TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata(BaseModel):
    __test__ = False
    annotation_result_id: str
    """The ID of the associated annotation result."""

    completion_tokens: int

    llm_reasoning: str
    """The reasoning the LLM gave for the annotation it provided."""

    prompt_tokens: int

    time_elapsed_s: int
    """The time elapsed to generate this annotation in seconds."""

    cost: Optional[int] = None
    """The cost of the annotation in cents."""


class TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResult(BaseModel):
    __test__ = False
    id: str
    """The unique identifier of the entity."""

    annotation_type: Literal["llm_auto", "human"]
    """An enumeration."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    question_id: str

    selected_choice: object
    """The selected choices(s) for the annotation result, in JSON form.

    For categorical questions, this is an object or list of objects (depending on if
    multiple selections are allowed). For free text questions, this is a string.
    """

    test_case_result_lineage_id: str

    llm_auto_eval_metadata: Optional[
        TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata
    ] = None


class TestCaseResultGenerationTestCaseResultResponseExpandedTaskAssignedTo(BaseModel):
    __test__ = False
    id: str

    email: str

    first_name: Optional[str] = None

    last_name: Optional[str] = None


class TestCaseResultGenerationTestCaseResultResponseExpandedTask(BaseModel):
    __test__ = False
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    priority: int

    status: Literal["PENDING", "COMPLETED"]
    """An enumeration."""

    task_entity_id: str

    task_entity_parent_id: str

    task_type: Literal["EVALUATION_ANNOTATION"]
    """An enumeration."""

    assigned_to: Optional[TestCaseResultGenerationTestCaseResultResponseExpandedTaskAssignedTo] = None

    assignment_expires_at: Optional[datetime] = None
    """The date and time when the task assignment expires in ISO format."""

    task_entity: Optional[test_case_result.TestCaseResult] = None
    """The entity that the task is associated with."""


class TestCaseResultGenerationTestCaseResultResponseExpanded(BaseModel):
    __test__ = False
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset: EvaluationDataset

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: ResultSchemaGeneration

    test_case_id: str

    test_case_version: TestCase

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    annotation_results: Optional[List[TestCaseResultGenerationTestCaseResultResponseExpandedAnnotationResult]] = None

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    result: Optional[object] = None

    task: Optional[TestCaseResultGenerationTestCaseResultResponseExpandedTask] = None

    test_case_evaluation_data_schema: Optional[Literal["GENERATION"]] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationDataGenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationData(BaseModel):
    __test__ = False
    generation_output: Union[
        str,
        float,
        List[FlexibleChunk],
        List[FlexibleMessage],
        List[object],
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]],
        object,
    ]

    generation_extra_info: Optional[
        TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationDataGenerationExtraInfo
    ] = None


class TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata(BaseModel):
    __test__ = False
    annotation_result_id: str
    """The ID of the associated annotation result."""

    completion_tokens: int

    llm_reasoning: str
    """The reasoning the LLM gave for the annotation it provided."""

    prompt_tokens: int

    time_elapsed_s: int
    """The time elapsed to generate this annotation in seconds."""

    cost: Optional[int] = None
    """The cost of the annotation in cents."""


class TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResult(BaseModel):
    __test__ = False
    id: str
    """The unique identifier of the entity."""

    annotation_type: Literal["llm_auto", "human"]
    """An enumeration."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    question_id: str

    selected_choice: object
    """The selected choices(s) for the annotation result, in JSON form.

    For categorical questions, this is an object or list of objects (depending on if
    multiple selections are allowed). For free text questions, this is a string.
    """

    test_case_result_lineage_id: str

    llm_auto_eval_metadata: Optional[
        TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResultLlmAutoEvalMetadata
    ] = None


class TestCaseResultFlexibleTestCaseResultResponseExpandedTaskAssignedTo(BaseModel):
    __test__ = False
    id: str

    email: str

    first_name: Optional[str] = None

    last_name: Optional[str] = None


class TestCaseResultFlexibleTestCaseResultResponseExpandedTask(BaseModel):
    __test__ = False
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    priority: int

    status: Literal["PENDING", "COMPLETED"]
    """An enumeration."""

    task_entity_id: str

    task_entity_parent_id: str

    task_type: Literal["EVALUATION_ANNOTATION"]
    """An enumeration."""

    assigned_to: Optional[TestCaseResultFlexibleTestCaseResultResponseExpandedTaskAssignedTo] = None

    assignment_expires_at: Optional[datetime] = None
    """The date and time when the task assignment expires in ISO format."""

    task_entity: Optional[test_case_result.TestCaseResult] = None
    """The entity that the task is associated with."""


class TestCaseResultFlexibleTestCaseResultResponseExpanded(BaseModel):
    __test__ = False
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset: EvaluationDataset

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: TestCaseResultFlexibleTestCaseResultResponseExpandedTestCaseEvaluationData

    test_case_id: str

    test_case_version: TestCase

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    annotation_results: Optional[List[TestCaseResultFlexibleTestCaseResultResponseExpandedAnnotationResult]] = None

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    result: Optional[object] = None

    task: Optional[TestCaseResultFlexibleTestCaseResultResponseExpandedTask] = None

    test_case_evaluation_data_schema: Optional[Literal["FLEXIBLE"]] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


TestCaseResult: TypeAlias = Annotated[
    Union[TestCaseResultGenerationTestCaseResultResponseExpanded, TestCaseResultFlexibleTestCaseResultResponseExpanded],
    PropertyInfo(discriminator="test_case_evaluation_data_schema"),
]


class EvaluationWithViews(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    completed_test_case_result_count: int
    """The number of test case results that have been completed for the evaluation"""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    description: str

    name: str

    status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    total_test_case_result_count: int
    """The total number of test case results for the evaluation"""

    annotation_config: Optional[AnnotationConfig] = None
    """Annotation configuration for tasking"""

    application_spec: Optional[ApplicationSpec] = None

    application_variant_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    async_jobs: Optional[List[AsyncJob]] = None

    completed_at: Optional[datetime] = None
    """
    The date and time that all test case results for the evaluation were completed
    for the evaluation in ISO format.
    """

    evaluation_config: Optional[object] = None

    evaluation_config_expanded: Optional[EvaluationConfigExpanded] = None

    evaluation_config_id: Optional[str] = None
    """The ID of the associated evaluation config."""

    question_id_to_annotation_config: Optional[Dict[str, AnnotationConfig]] = None
    """Specifies the annotation configuration to use for specific questions."""

    tags: Optional[object] = None

    test_case_results: Optional[List[TestCaseResult]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CharacterChunkingStrategyConfigParam"]


class CharacterChunkingStrategyConfigParam(TypedDict, total=False):
    strategy: Required[Literal["character"]]

    chunk_overlap: int
    """Number of characters to overlap between chunks.

    If not specified, an overlap of 200 will be used. For example if the chunk size
    is 3 and the overlap size is 1, and the text to chunk is 'abcde', the chunks
    will be 'abc', 'cde'.
    """

    chunk_size: int
    """Maximum number of characters in each chunk.

    If not specified, a chunk size of 1000 will be used.
    """

    separator: str
    """Character designating breaks in input data.

    Text data will first be split into sections by this separator, then each section
    will be split into chunks of size `chunk_size`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["ApplicationEdge"]


class ApplicationEdge(BaseModel):
    from_field: str

    from_node: str

    to_field: str

    to_node: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import interaction_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.interaction_create_response import InteractionCreateResponse

__all__ = ["InteractionsResource", "AsyncInteractionsResource"]


class InteractionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InteractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return InteractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InteractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return InteractionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        application_variant_id: str,
        input: interaction_create_params.Input,
        output: interaction_create_params.Output,
        start_timestamp: Union[str, datetime],
        duration_ms: int | NotGiven = NOT_GIVEN,
        operation_metadata: object | NotGiven = NOT_GIVEN,
        operation_status: Literal["SUCCESS", "ERROR"] | NotGiven = NOT_GIVEN,
        thread_id: str | NotGiven = NOT_GIVEN,
        trace_spans: Iterable[interaction_create_params.TraceSpan] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InteractionCreateResponse:
        """
        Create Application Interaction

        Args:
          application_variant_id: Identifier for the application variant that performed this interaction.

          input: Represents the input of an interaction.

          output: Represents the output of an interaction.

          start_timestamp: Timestamp marking the start of the interaction.

          duration_ms: Duration of the interaction in milliseconds.

          operation_metadata: Optional metadata related to the operation, including custom or predefined keys.

          operation_status: Enum representing the status of an operation.

          thread_id: Optional UUID identifying the conversation thread associated with the
              interaction.The interaction will be associated with the thread if the id
              represents an existing thread.If the thread with the specified id is not found,
              a new thread will be created.

          trace_spans: List of trace spans associated with the interaction.These spans provide insight
              into the individual steps taken by nodes involved in generating the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/interactions/",
            body=maybe_transform(
                {
                    "application_variant_id": application_variant_id,
                    "input": input,
                    "output": output,
                    "start_timestamp": start_timestamp,
                    "duration_ms": duration_ms,
                    "operation_metadata": operation_metadata,
                    "operation_status": operation_status,
                    "thread_id": thread_id,
                    "trace_spans": trace_spans,
                },
                interaction_create_params.InteractionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InteractionCreateResponse,
        )


class AsyncInteractionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInteractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInteractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInteractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncInteractionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        application_variant_id: str,
        input: interaction_create_params.Input,
        output: interaction_create_params.Output,
        start_timestamp: Union[str, datetime],
        duration_ms: int | NotGiven = NOT_GIVEN,
        operation_metadata: object | NotGiven = NOT_GIVEN,
        operation_status: Literal["SUCCESS", "ERROR"] | NotGiven = NOT_GIVEN,
        thread_id: str | NotGiven = NOT_GIVEN,
        trace_spans: Iterable[interaction_create_params.TraceSpan] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InteractionCreateResponse:
        """
        Create Application Interaction

        Args:
          application_variant_id: Identifier for the application variant that performed this interaction.

          input: Represents the input of an interaction.

          output: Represents the output of an interaction.

          start_timestamp: Timestamp marking the start of the interaction.

          duration_ms: Duration of the interaction in milliseconds.

          operation_metadata: Optional metadata related to the operation, including custom or predefined keys.

          operation_status: Enum representing the status of an operation.

          thread_id: Optional UUID identifying the conversation thread associated with the
              interaction.The interaction will be associated with the thread if the id
              represents an existing thread.If the thread with the specified id is not found,
              a new thread will be created.

          trace_spans: List of trace spans associated with the interaction.These spans provide insight
              into the individual steps taken by nodes involved in generating the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/interactions/",
            body=await async_maybe_transform(
                {
                    "application_variant_id": application_variant_id,
                    "input": input,
                    "output": output,
                    "start_timestamp": start_timestamp,
                    "duration_ms": duration_ms,
                    "operation_metadata": operation_metadata,
                    "operation_status": operation_status,
                    "thread_id": thread_id,
                    "trace_spans": trace_spans,
                },
                interaction_create_params.InteractionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InteractionCreateResponse,
        )


class InteractionsResourceWithRawResponse:
    def __init__(self, interactions: InteractionsResource) -> None:
        self._interactions = interactions

        self.create = to_raw_response_wrapper(
            interactions.create,
        )


class AsyncInteractionsResourceWithRawResponse:
    def __init__(self, interactions: AsyncInteractionsResource) -> None:
        self._interactions = interactions

        self.create = async_to_raw_response_wrapper(
            interactions.create,
        )


class InteractionsResourceWithStreamingResponse:
    def __init__(self, interactions: InteractionsResource) -> None:
        self._interactions = interactions

        self.create = to_streamed_response_wrapper(
            interactions.create,
        )


class AsyncInteractionsResourceWithStreamingResponse:
    def __init__(self, interactions: AsyncInteractionsResource) -> None:
        self._interactions = interactions

        self.create = async_to_streamed_response_wrapper(
            interactions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ChatThread"]


class ChatThread(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    title: str

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.model_groups import usage_statistic_retrieve_params
from ...types.shared.model_usage import ModelUsage

__all__ = ["UsageStatisticsResource", "AsyncUsageStatisticsResource"]


class UsageStatisticsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageStatisticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return UsageStatisticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageStatisticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return UsageStatisticsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        model_group_id: str,
        *,
        chunks: int,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelUsage:
        """
        Get Model usage for a group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return self._get(
            f"/v4/model-groups/{model_group_id}/usage-statistics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "chunks": chunks,
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    usage_statistic_retrieve_params.UsageStatisticRetrieveParams,
                ),
            ),
            cast_to=ModelUsage,
        )


class AsyncUsageStatisticsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageStatisticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageStatisticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageStatisticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncUsageStatisticsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        model_group_id: str,
        *,
        chunks: int,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelUsage:
        """
        Get Model usage for a group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return await self._get(
            f"/v4/model-groups/{model_group_id}/usage-statistics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "chunks": chunks,
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    usage_statistic_retrieve_params.UsageStatisticRetrieveParams,
                ),
            ),
            cast_to=ModelUsage,
        )


class UsageStatisticsResourceWithRawResponse:
    def __init__(self, usage_statistics: UsageStatisticsResource) -> None:
        self._usage_statistics = usage_statistics

        self.retrieve = to_raw_response_wrapper(
            usage_statistics.retrieve,
        )


class AsyncUsageStatisticsResourceWithRawResponse:
    def __init__(self, usage_statistics: AsyncUsageStatisticsResource) -> None:
        self._usage_statistics = usage_statistics

        self.retrieve = async_to_raw_response_wrapper(
            usage_statistics.retrieve,
        )


class UsageStatisticsResourceWithStreamingResponse:
    def __init__(self, usage_statistics: UsageStatisticsResource) -> None:
        self._usage_statistics = usage_statistics

        self.retrieve = to_streamed_response_wrapper(
            usage_statistics.retrieve,
        )


class AsyncUsageStatisticsResourceWithStreamingResponse:
    def __init__(self, usage_statistics: AsyncUsageStatisticsResource) -> None:
        self._usage_statistics = usage_statistics

        self.retrieve = async_to_streamed_response_wrapper(
            usage_statistics.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .evaluations.test_case_result import TestCaseResult

__all__ = ["Task"]


class Task(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    priority: int

    status: Literal["PENDING", "COMPLETED"]
    """An enumeration."""

    task_entity_id: str

    task_entity_parent_id: str

    task_type: Literal["EVALUATION_ANNOTATION"]
    """An enumeration."""

    assigned_to: Optional[str] = None
    """The ID of the user that the task is assigned to."""

    assignment_expires_at: Optional[datetime] = None
    """The date and time when the task assignment expires in ISO format."""

    task_entity: Optional[TestCaseResult] = None
    """The entity that the task is associated with."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import ExecuteAgentResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_execute(self, client: SGPClient) -> None:
        agent = client.agents.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        )
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    def test_method_execute_with_all_params(self, client: SGPClient) -> None:
        agent = client.agents.execute(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            tools=[
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
            ],
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
        )
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_execute(self, client: SGPClient) -> None:
        response = client.agents.with_raw_response.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_execute(self, client: SGPClient) -> None:
        with client.agents.with_streaming_response.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAgents:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_execute(self, async_client: AsyncSGPClient) -> None:
        agent = await async_client.agents.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        )
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    async def test_method_execute_with_all_params(self, async_client: AsyncSGPClient) -> None:
        agent = await async_client.agents.execute(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            tools=[
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {
                        "type": "object",
                        "properties": {
                            "foo": {
                                "description": "description",
                                "type": "string",
                                "default": "default",
                                "examples": ["string", "string", "string"],
                            }
                        },
                    },
                    "description": "description",
                    "name": "name",
                },
            ],
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
        )
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_execute(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.agents.with_raw_response.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_execute(self, async_client: AsyncSGPClient) -> None:
        async with async_client.agents.with_streaming_response.execute(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            tools=[
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
                {
                    "arguments": {"type": "object"},
                    "description": "description",
                    "name": "name",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(ExecuteAgentResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AzureBlobStorageDataSourceAuthConfigParam"]


class AzureBlobStorageDataSourceAuthConfigParam(TypedDict, total=False):
    blob_sas_token: Required[str]
    """Shared Access Signature token for the Azure Blob Storage container"""

    source: Required[Literal["AzureBlobStorage"]]

    encrypted: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .history_list_params import HistoryListParams as HistoryListParams
from .paginated_test_case_results import PaginatedTestCaseResults as PaginatedTestCaseResults



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["TimeseriesData", "Value"]


class Value(BaseModel):
    label: str

    value: List[float]


class TimeseriesData(BaseModel):
    bins: List[datetime]

    values: List[Value]

    unit: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["QuestionCreateParams"]


class QuestionCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    prompt: Required[str]

    title: Required[str]

    type: Required[Literal["categorical", "free_text"]]
    """An enumeration."""

    choices: Iterable[object]
    """List of choices for the question. Required for CATEGORICAL questions."""

    conditions: Iterable[object]
    """Conditions for the question to be shown."""

    dropdown: bool
    """Whether the question is displayed as a dropdown in the UI."""

    multi: bool
    """Whether the question allows multiple answers."""

    required: bool
    """Whether the question is required."""
